@@ -692,7 +692,7 @@ module Coff = struct
       | _ -> raise Exit
     and aux1 i0 i = if i = l then (String.sub s i0 (i - i0), [])::[]
     else match s.[i] with
-      | 'a'..'z' | 'A'..'Z' -> aux1 i0 (i+1)
+      | 'a'..'z' | 'A'..'Z' | '-' -> aux1 i0 (i+1)
       | ' ' -> (String.sub s i0 (i - i0), []) :: aux0 (i+1)
       | ':' -> aux2 (String.sub s i0 (i - i0)) [] (i+1)
       | _   -> raise Exit
@@ -1038,31 +1038,31 @@ end
 
 
 module Stacksize = struct
-  let set_stack_reserve filename reserve =
+  let set_stack_reserve filename size_of_stack_reserve =
     let filename =
       if not (Sys.file_exists filename) && (Sys.file_exists (filename ^ ".exe")) then filename ^ ".exe"
       else filename
     in
     let ic = open_in_bin filename in
-    let hdr_offset = int16 (read ic 0x3c 2) 0 in
-    let pe_signature = read_str ic hdr_offset 4 in
+    (* Read e_lfanew from the IMAGE_DOS_HEADER (see winnt.h) *)
+    let pe_signature_offset = int32_ (read ic 0x3c 4) 0 in
+    let pe_signature = read_str ic pe_signature_offset 4 in
     assert(pe_signature = "PE\000\000");
-    let coff_hdr = read ic 0 20 in
-    let opthdr_size = int16 coff_hdr 16 in
-    let opthdr = read ic (hdr_offset + 24) opthdr_size in
-    let machine =
-      match int16 opthdr 0 with
-      | 0x10b -> `x86
-      | 0x20b -> `x64
-      | magic -> Printf.ksprintf failwith "Cannot determine image target (magic = %x)." magic
+    let coff_file_header = read ic (pe_signature_offset + 4) 20 in
+    let size_of_optional_header = int16 coff_file_header 16 in
+    let optional_header = read ic (pe_signature_offset + 4 + 20) size_of_optional_header in
+    let pe_format =
+      match int16 optional_header 0 with
+      | 0x10b -> `PE32
+      | 0x20b -> `PE32plus
+      | magic -> Printf.ksprintf failwith "Cannot determine PE format (magic = %x)." magic
     in
-    let reserve_offset = hdr_offset + 24 + 72 in
-(*    Printf.printf "current stack reserve %ld\n%!" (int32 opthdr 72); *)
+    let size_of_stack_reserve_offset = pe_signature_offset + 4 + 20 + 72 in
     close_in ic;
 
     let oc = open_out_gen [Open_wronly; Open_binary] 0x777 filename in
-    seek_out oc reserve_offset;
-    emit_int32 oc reserve;
-    if machine = `x64 then emit_int32 oc 0l;
+    seek_out oc size_of_stack_reserve_offset;
+    emit_int32 oc size_of_stack_reserve;
+    if pe_format = `PE32plus then emit_int32 oc 0l;
     close_out oc
 end
