@@ -1,9 +1,5 @@
 (* TEST
-   include unix
-   modules = "test7_.c"
-   * libunix
-   ** bytecode
-   ** native
+ modules = "test7_.c";
 *)
 
 (* Tests nested calls from C (main C) to OCaml (main OCaml) to C (caml_to_c) to
